/* ipuz-clue.h
 *
 * Copyright 2021 Jonathan Blandford <jrb@gnome.org>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>
#include <json-glib/json-glib.h>
#include <ipuz-enumeration.h>

G_BEGIN_DECLS


typedef enum _IPuzClueDirection
{
  IPUZ_CLUE_DIRECTION_ACROSS,
  IPUZ_CLUE_DIRECTION_DOWN,
  IPUZ_CLUE_DIRECTION_DIAGONAL,
  IPUZ_CLUE_DIRECTION_DIAGONAL_UP,
  IPUZ_CLUE_DIRECTION_DIAGONAL_DOWN_LEFT,
  IPUZ_CLUE_DIRECTION_DIAGONAL_UP_LEFT,
  IPUZ_CLUE_DIRECTION_ZONES,
  IPUZ_CLUE_DIRECTION_CLUES,
  IPUZ_CLUE_DIRECTION_NONE,
} IPuzClueDirection;


typedef struct _IPuzCellCoord IPuzCellCoord;
struct _IPuzCellCoord
{
  guint row;
  guint column;
};

GType            ipuz_cell_coord_get_type    (void) G_GNUC_CONST;
IPuzCellCoord   *ipuz_cell_coord_dup         (const IPuzCellCoord *coord);

typedef struct _IPuzClue IPuzClue;
struct _IPuzClue
{
  /* Just fields for crosswords, to start. */
  gint number;
  gchar *label;
  gchar *clue_text;
  IPuzClueDirection direction;
  GArray *cells; /* IPuzCellCoord */

  /* < Private > */
  IPuzEnumeration *enumeration;
  gboolean cells_set; /* TRUE, if cells were loaded vs. calculated */
};


#define IPUZ_TYPE_CLUE (ipuz_clue_get_type ())


GType              ipuz_clue_get_type           (void);
IPuzClue          *ipuz_clue_new                (void);
IPuzClue          *ipuz_clue_copy               (const IPuzClue    *clue);
void               ipuz_clue_free               (IPuzClue          *clue);
gboolean           ipuz_clue_compare            (const IPuzClue    *clue1,
                                                 const IPuzClue    *clue2);
void               ipuz_clue_build              (IPuzClue          *clue,
                                                 JsonBuilder       *builder);
IPuzClueDirection  ipuz_clue_direction_switch   (IPuzClueDirection  direction);
gint               ipuz_clue_get_number         (IPuzClue          *clue);
void               ipuz_clue_set_number         (IPuzClue          *clue,
                                                 gint               number);
const gchar       *ipuz_clue_get_label          (IPuzClue          *clue);
void               ipuz_clue_set_label          (IPuzClue          *clue,
                                                 const gchar       *label);
const gchar       *ipuz_clue_get_clue_text      (IPuzClue          *clue);
void               ipuz_clue_set_clue_text      (IPuzClue          *clue,
                                                 const gchar       *clue_text);
IPuzEnumeration   *ipuz_clue_get_enumeration    (IPuzClue          *clue);
void               ipuz_clue_set_enumeration    (IPuzClue          *clue,
                                                 IPuzEnumeration   *enumeration);
IPuzClueDirection  ipuz_clue_get_direction      (IPuzClue          *clue);
void               ipuz_clue_set_direction      (IPuzClue          *clue,
                                                 IPuzClueDirection  direction);
const GArray      *ipuz_clue_get_cells          (IPuzClue          *clue);
void               ipuz_clue_append_cell        (IPuzClue          *clue,
                                                 IPuzCellCoord      coord);
void               ipuz_clue_get_first_cell     (IPuzClue          *clue,
                                                 IPuzCellCoord     *coord);
void               ipuz_clue_get_last_cell      (IPuzClue          *clue,
                                                 IPuzCellCoord     *coord);
gboolean           ipuz_clue_contains_cell      (IPuzClue          *clue,
                                                 IPuzCellCoord      coord);
IPuzClue          *ipuz_clue_new_from_json      (JsonNode          *node);
void               ipuz_clue_ensure_enumeration (IPuzClue          *clue);


G_END_DECLS
