/* ipuz-crossword.h
 *
 * Copyright 2021 Jonathan Blandford <jrb@gnome.org>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>
#include <libipuz/ipuz-puzzle.h>
#include <libipuz/ipuz-board.h>
#include <libipuz/ipuz-clue.h>
#include <libipuz/ipuz-guesses.h>

G_BEGIN_DECLS


#define IPUZ_TYPE_CROSSWORD (ipuz_crossword_get_type ())
G_DECLARE_FINAL_TYPE (IPuzCrossword, ipuz_crossword, IPUZ, CROSSWORD, IPuzPuzzle);


typedef enum
{
  IPUZ_SYMMETRY_NONE,    /* @ Freeform puzzle @ */
  IPUZ_SYMMETRY_QUARTER, /* @ 90° Rotational Symmetry around the center. Only works on square boards. */
  IPUZ_SYMMETRY_HALF,    /* @ 180° Rotational Symmetry around the center*/
} IPuzSymmetry;


IPuzCrossword *ipuz_crossword_new                 (void);
IPuzCrossword *ipuz_crossword_deep_copy           (IPuzCrossword     *xword);
guint          ipuz_crossword_get_width           (IPuzCrossword     *xword);
guint          ipuz_crossword_get_height          (IPuzCrossword     *xword);
void           ipuz_crossword_set_size            (IPuzCrossword     *xword,
                                                   gint               width,
                                                   gint               height);
IPuzBoard     *ipuz_crossword_get_board           (IPuzCrossword     *xword);
gboolean       ipuz_crossword_set_guesses         (IPuzCrossword     *xword,
                                                   IPuzGuesses       *guesses);
IPuzGuesses   *ipuz_crossword_get_guesses         (IPuzCrossword     *xword);
IPuzCell      *ipuz_crossword_get_cell            (IPuzCrossword     *xword,
                                                   IPuzCellCoord      coord);
GArray        *ipuz_crossword_get_clues           (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction);
IPuzClue      *ipuz_crossword_get_clue            (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction,
                                                   guint              index);
gchar         *ipuz_crossword_get_clue_string     (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction,
                                                   gint               index);
gchar         *ipuz_crossword_get_guess_string    (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction,
                                                   gint               number);
gboolean       ipuz_crossword_clue_guessed        (IPuzCrossword     *xword,
                                                   IPuzClue          *clue,
                                                   gboolean          *correct);
guint          ipuz_crossword_get_n_clues         (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction);
guint          ipuz_crossword_find_clue           (IPuzCrossword     *xword,
                                                   IPuzClue          *clue);
IPuzClue      *ipuz_crossword_find_clue_by_number (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction,
                                                   gint               number);
IPuzClue      *ipuz_crossword_find_clue_by_label  (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction,
                                                   const char        *label);
IPuzClue      *ipuz_crossword_find_clue_by_coord  (IPuzCrossword     *xword,
                                                   IPuzClueDirection  direction,
                                                   IPuzCellCoord      coord);
gboolean       ipuz_crossword_game_won            (IPuzCrossword     *xword);


/* Mutators */
void           ipuz_crossword_calculate_clues     (IPuzCrossword     *xword);
void           ipuz_crossword_set_symmetry        (IPuzCrossword     *xword,
                                                   IPuzSymmetry       symmetry);
IPuzSymmetry   ipuz_crossword_get_symmetry        (IPuzCrossword     *xword);
void           ipuz_crossword_set_cell_type       (IPuzCrossword     *xword,
                                                   IPuzCellCoord      coord,
                                                   IPuzCellCellType   cell_type);

/* Public functions */
void           ipuz_crossword_print               (IPuzCrossword     *xword);


G_END_DECLS
